import os
from setuptools import setup
from jam import version

cur_dir = os.getcwd()

setup(
    name='jam.py',
    version=version(),
    url='http://jam-py.com/',
    author='Andrew Yushev',
    author_email='yushevaa@gmail.com',
    description=('Jam.py framework is the fastest way to create a web database application.'),
    license='BSD',
    packages=['jam', 'jam.lang', 'jam.db', 'jam.third_party', 'jam.third_party.werkzeug',
        'jam.third_party.werkzeug.contrib', 'jam.third_party.werkzeug.debug',
        'jam.third_party.slimit', 'jam.third_party.slimit.ply',
        'jam.third_party.slimit.visitors'],
    package_data={'jam': ['admin.html', 'js/*.js', 'js/ace/*.js',
        'img/*.*', 'css/*.*', 'project/*.*', 'project/css/*.*',
        'project/js/*.*', 'project/img/*.*', 'third_party/werkzeug/debug/shared/*.*']},
    scripts=['jam/bin/jam-project.py'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: JavaScript',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Database',
        'Topic :: Database :: Front-Ends'
    ],
)



# -*- coding: utf-8 -*-

from __future__ import with_statement
import sys
import json
import traceback

import common, db.db_modules as db_modules
from dataset import *

class SQL(object):

    def get_next_id(self, db_module=None):
        if db_module is None:
            db_module = self.task.db_module
        sql = db_module.next_sequence_value_sql(self.table_name)
        if sql:
            rec = self.task.execute_select(sql)
            if rec:
                if rec[0][0]:
                    return int(rec[0][0])

    def insert_sql(self, db_module):
        if self._deleted_flag:
            self._deleted_flag_field.set_data(0)
        sql = 'INSERT INTO "%s" (' % self.table_name
        row = []
        fields = ''
        values = ''
        index = 0
        for field in self.fields:
            if not (field.calculated or field.master_field):
                index += 1
                fields += '"%s", ' % field.field_name
                values +=  '%s, ' % db_module.value_literal(index)
                value = (field.raw_value, field.data_type)
                row.append(value)
        fields = fields[:-2]
        values = values[:-2]
        sql = db_module.set_case('INSERT INTO "%s" (%s) VALUES ' % \
            (self.table_name, fields)) + '(' + values + ')'
        return sql, row

    def update_sql(self, db_module):
        row = []
        command = db_module.set_case('UPDATE "%s" SET ' % self.table_name)
        fields = ''
        index = 0
        for field in self.fields:
            if not (field.calculated or field.master_field):
                index += 1
                fields += '"%s"=%s, ' % \
                    (db_module.set_case(field.field_name),
                    db_module.value_literal(index))
                value = (field.raw_value, field.data_type)
                if field.field_name == self._deleted_flag:
                    value = (0, field.data_type)
                row.append(value)
        fields = fields[:-2]
        where = db_module.set_case(' WHERE %s = %s' % \
            (self._primary_key, self._primary_key_field.value))
        return command + fields + where, row

    def delete_sql(self, db_module):
        soft_delete = self.soft_delete
        if self.master:
            soft_delete = self.master.soft_delete
        if soft_delete:
            sql = 'UPDATE %s SET %s = 1 WHERE %s = %s' % \
                (self.table_name, self._deleted_flag,
                self._primary_key, self._primary_key_field.value)
        else:
            sql = 'DELETE FROM %s WHERE %s = %s' % \
                (self.table_name, self._primary_key,
                self._primary_key_field.value)
        return db_module.set_case(sql)

    def apply_sql(self, user_info=None, priv=None, db_module=None):

        def get_sql(item, db_module):
            if item.master:
                item._master_id_field.set_data(item.master.ID)
                item._master_rec_id_field.set_data(item.master._primary_key_field.value)
            if item.record_status == common.RECORD_INSERTED:
                if not item.master:
                    if priv and not priv['can_create']:
                        raise Exception, self.task.lang['cant_create'] % self.item_caption
                sql, param = item.insert_sql(db_module)
            elif item.record_status == common.RECORD_MODIFIED:
                if not item.master:
                    if priv and not priv['can_edit']:
                        raise Exception, self.task.lang['cant_edit'] % self.item_caption
                sql, param = item.update_sql(db_module)
            elif item.record_status == common.RECORD_DETAILS_MODIFIED:
                sql, param = '', None
            elif item.record_status == common.RECORD_DELETED:
                if not item.master:
                    if priv and not priv['can_delete']:
                        raise Exception, self.task.lang['cant_delete'] % self.item_caption
                sql = item.delete_sql(db_module)
                param = None
            else:
                raise Exception, u'apply_sql - invalid %s record_status %s, record: %s' % (item.item_name, item.record_status, item._dataset[item.rec_no])
            primary_key_index = item.fields.index(item._primary_key_field)
            master_rec_id_index = None
            if item.master:
                master_rec_id_index = item.fields.index(item._master_rec_id_field)
            info = {
                'ID': item.ID,
                'table_name': item.table_name,
                'status': item.record_status,
                'primary_key': item._primary_key_field.value,
                'primary_key_index': primary_key_index,
                'log_id': item.get_rec_info()[common.REC_CHANGE_ID],
                'master_rec_id_index': master_rec_id_index
                }
            h_sql, h_params, h_table_name = get_history_sql(item, user_info, db_module)
            return sql, param, info, h_sql, h_params, h_table_name

        def delete_detail_sql(item, detail, db_module):
            if item.soft_delete:
                sql = 'UPDATE %s SET %s = 1 WHERE %s = %s AND %s = %s' % \
                    (detail.table_name, detail._deleted_flag, detail._master_id, \
                    item.ID, detail._master_rec_id, self._primary_key_field.value)
            else:
                sql = 'DELETE FROM %s WHERE %s = %s AND %s = %s' % \
                    (detail.table_name, detail._master_id, item.ID, \
                    detail._master_rec_id, self._primary_key_field.value)
            h_sql, h_params, h_table_name = get_history_sql(item, user_info, db_module)
            return db_module.set_case(sql), None, None, h_sql, h_params, h_table_name

        def get_history_sql(item, user_info, db_module):
            h_sql = None
            h_params = None
            h_table_name = None
            if item.task.history_item and item.keep_history or (item.master and item.master.keep_history):
                h_table_name = item.task.history_item.table_name
                changes = None
                user = None
                if user_info:
                    user = user_info['user_name']
                if item.record_status != common.RECORD_DELETED:
                    old_rec = item.get_rec_info()[3]
                    new_rec = item._dataset[item.rec_no]
                    f_list = []
                    for f in item.fields:
                        old = None
                        if old_rec:
                            old = old_rec[f.bind_index]
                        new = new_rec[f.bind_index]
                        if old != new:
                            f_list.append([f.ID, old, new])
                    d_list = []
                    if not item.master:
                        for detail in item.details:
                            for d in detail:
                                d_list.append([d.ID, d._primary_key_field.value, d.record_status])
                    changes = (json.dumps([f_list, d_list], default=common.json_defaul_handler), common.BLOB)
                h_fields = ['id', 'item_id', 'item_rec_id', 'operation', 'changes', 'user', 'date']
                h_params = [None, item.ID, item._primary_key_field.value, item.record_status, changes, user, datetime.datetime.now()]
                if item.task.history_item._deleted_flag:
                    h_fields.append('deleted')
                    h_params.append(0)
                index = 0
                fields = ''
                values = ''
                for f in h_fields:
                    index += 1
                    fields += '"%s", ' % f
                    values +=  '%s, ' % db_module.value_literal(index)
                fields = fields[:-2]
                values = values[:-2]
                h_sql = db_module.set_case('INSERT INTO "%s" (%s) VALUES ' % \
                    (item.task.history_item.table_name, fields)) + '(' + values + ')'
            return h_sql, h_params, h_table_name

        def generate_sql(item, db_module, result):
            ID, sql = result
            for it in item:
                details = []
                sql.append((get_sql(it, db_module), details))
                for detail in item.details:
                    detail_sql = []
                    detail_result = (str(detail.ID), detail_sql)
                    details.append(detail_result)
                    if item.record_status == common.RECORD_DELETED:
                        detail_sql.append((delete_detail_sql(item, detail, db_module), []))
                    else:
                        generate_sql(detail, db_module, detail_result)

        if db_module is None:
            db_module = self.task.db_module
        result = (self.ID, [])
        generate_sql(self, db_module, result)
        return {'delta': result}

    def table_alias(self):
        return '"%s"' % self.table_name

    def ref_table_alias(self, field):
        if field.master_field:
            return '%s_%d' % (field.lookup_item.table_name, field.master_field.ID)
        else:
            return '%s_%d' % (field.lookup_item.table_name, field.ID)

    def fields_clause(self, query, fields, db_module=None):
        if db_module is None:
            db_module = self.task.db_module
        funcs = query.get('__funcs')
        if funcs:
            functions = {}
            for key, value in funcs.iteritems():
                functions[key.upper()] = value
        sql = ''
        for field in fields:
            if field.master_field:
                pass
            elif field.calculated:
                sql += 'NULL AS "%s", ' % field.field_name
            else:
                field_sql = '%s."%s"' % (self.table_alias(), field.field_name)
                if funcs:
                    func = functions.get(field.field_name.upper())
                    if func:
                        field_sql = '%s(%s) AS "%s"' % (func, field_sql, field.field_name)
                sql += field_sql + ', '
        if query['__expanded']:
            for field in fields:
                if field.lookup_item:
                    field_sql = '%s."%s" %s %s_LOOKUP' % \
                    (self.ref_table_alias(field), field.lookup_field, db_module.FIELD_AS, field.field_name)
                    if funcs:
                        func = functions.get(field.field_name.upper())
                        if func:
                            field_sql = '%s(%s."%s") %s %s_LOOKUP' % \
                            (func, self.ref_table_alias(field), field.lookup_field, db_module.FIELD_AS, field.field_name)
                    sql += field_sql + ', '
        sql = sql[:-2]
        return db_module.set_case(sql)

    def from_clause(self, query, fields, db_module=None):
        if db_module is None:
            db_module = self.task.db_module
        result = db_module.FROM % (self.table_name, self.table_alias())
        if query['__expanded']:
            for field in fields:
                if field.lookup_item and not field.master_field:
                    primary_key_field_name = field.lookup_item._primary_key
                    result = '(' + result
                    result += ' ' + db_module.LEFT_OUTER_JOIN % (field.lookup_item.table_name, self.ref_table_alias(field))
                    result += ' ON %s."%s"' % (self.table_alias(), field.field_name)
                    result += ' = %s."%s")'  % (self.ref_table_alias(field), primary_key_field_name)
        return db_module.set_case(result)

    def where_clause(self, query, db_module=None):

        def get_filter_sign(filter_type, value=None):
            result = common.FILTER_SIGN[filter_type]
            if filter_type == common.FILTER_ISNULL:
                if value:
                    result = 'IS NULL'
                else:
                    result = 'IS NOT NULL'
            if (result == 'LIKE'):
                result = db_module.LIKE
            return result

        def convert_field_value(field, value, filter_type=None):
            data_type = field.data_type
            if data_type == common.DATE:
                if type(value) in (str, unicode):
                    result = value
                else:
                    result = value.strftime('%Y-%m-%d')
                return db_module.cast_date(result)# "cast('" + result + "' AS DATE)"
            elif data_type == common.DATETIME:
                if type(value) in (str, unicode):
                    result = value
                else:
                    result = value.strftime('%Y-%m-%d %H:%M')
                result = db_module.cast_datetime(result)#"cast('" + result + "' AS TIMESTAMP)"
                return result
            elif data_type == common.INTEGER:
                if type(value) == int or type(value) in [str, unicode] and value.isdigit():
                    return str(value)
                else:
                    if filter_type and filter_type in [common.FILTER_CONTAINS, common.FILTER_STARTWITH, common.FILTER_ENDWITH]:
                        return value
                    else:
                        return "'" + value + "'"
            elif data_type == common.BOOLEAN:
                if value:
                    return '1'
                else:
                    return '0'
            elif data_type == common.TEXT:
                if filter_type and filter_type in [common.FILTER_CONTAINS, common.FILTER_STARTWITH, common.FILTER_ENDWITH]:
                    return value
                else:
                    return "'" + value + "'"
            elif data_type in (common.FLOAT, common.CURRENCY):
                value = float(value)
                return str(value)
            else:
                return value

        def escape_search(value, esc_char):
            result = ''
            found = False
            for ch in value:
                if ch == "'":
                    ch = ch + ch
                elif ch in ['_', '%']:
                    ch = esc_char + ch
                    found = True
                result += ch
            return result, found

        def get_condition(field_name, filter_type, value):
            field = self._field_by_name(field_name)
            esc_char = '/'
            cond_field_name = db_module.set_case('%s."%s"' % (self.table_alias(), field_name))
            if type(value) == str:
                value = value.decode('utf-8')
            filter_sign = get_filter_sign(filter_type, value)
            cond_string = '%s %s %s'
            if filter_type in (common.FILTER_IN, common.FILTER_NOT_IN):
                lst = '('
                if len(value):
                    for it in value:
                        lst += convert_field_value(field, it) + ', '
                    value = lst[:-2] + ')'
                else:
                    value = '()'
            elif filter_type == common.FILTER_RANGE:
                value = convert_field_value(field, value[0]) + \
                    db_module.set_case(' AND ') + convert_field_value(field, value[1])
            elif filter_type == common.FILTER_ISNULL:
                value = ''
            else:
                value = convert_field_value(field, value, filter_type)
                if filter_type in [common.FILTER_CONTAINS, common.FILTER_STARTWITH, common.FILTER_ENDWITH]:
                    value, esc_found = escape_search(value, esc_char)
                    if field.lookup_item:
                        cond_field_name = db_module.set_case('%s."%s"' % (self.ref_table_alias(field), field.lookup_field))
                    if filter_type == common.FILTER_CONTAINS:
                        value = '%' + value + '%'
                    elif filter_type == common.FILTER_STARTWITH:
                        value = value + '%'
                    elif filter_type == common.FILTER_ENDWITH:
                        value = '%' + value
                    upper_function =  db_module.upper_function()
                    if upper_function:
                        cond_string = upper_function + '(%s) %s %s'
                        value = value.upper()
                    if esc_found:
                        value = "'" + value + db_module.set_case("' ESCAPE '") + esc_char + "'"
                    else:
                        value = "'" + value + "'"
            sql = cond_string % (cond_field_name, filter_sign, value)
            if field.data_type == common.BOOLEAN and value == '0':
                if filter_sign == '=':
                    sql = '(' + sql + ' OR %s IS NULL)' % cond_field_name
                elif filter_sign == '<>':
                    sql = '(' + sql + ' AND %s IS NOT NULL)' % cond_field_name
                else:
                    raise Exception, 'sql.py where_clause method: boolen field condition may give ambiguious results.'
            return sql

        if db_module is None:
            db_module = self.task.db_module
        result = ''
        conditions = []
        filters = query['__filters']
        deleted_in_filters = False
        if filters:
            for (field_name, filter_type, value) in filters:
                if not value is None:
                    if field_name == self._deleted_flag:
                        deleted_in_filters = True
                    if filter_type == common.FILTER_CONTAINS_ALL:
                        values = value.split()
                        for val in values:
                            conditions.append(get_condition(field_name, common.FILTER_CONTAINS, val))
                    elif filter_type in [common.FILTER_IN, common.FILTER_NOT_IN] and \
                        type(value) in [tuple, list] and len(value) == 0:
                        conditions.append(db_module.set_case('%s."%s" IN (NULL)' % (self.table_alias(), self._primary_key)))
                    else:
                        conditions.append(get_condition(field_name, filter_type, value))
        if not deleted_in_filters and self._deleted_flag:
            conditions.append(db_module.set_case('%s."%s"=0' % \
            (self.table_alias(), self._deleted_flag)))
        for sql in conditions:
            result += sql + db_module.set_case(' AND ')
        result = result[:-5]
        if result:
            result = db_module.set_case(' WHERE ') + result
        return result

    def group_clause(self, query, fields, db_module=None):
        if db_module is None:
            db_module = self.task.db_module
        group_fields = query.get('__group_by')
        result = ''
        if group_fields:
            for field_name in group_fields:
                field = self.field_by_name(field_name)
                result += '%s."%s", ' % (self.table_alias(), field_name)
                if field.lookup_item:
                    result += '%s_LOOKUP, ' % field.field_name
            if result:
                result = result[:-2]
                result = ' GROUP BY ' + result
            return db_module.set_case(result)
        else:
            return ''

    def order_clause(self, query, db_module=None):
        result = ''
        if query.get('__funcs') and not query.get('__group_by'):
            return result
        if db_module is None:
            db_module = self.task.db_module
        order_list = query.get('__order', [])
        for order in order_list:
            field = self._field_by_ID(order[0])
            if field:
                if query['__expanded'] and field.lookup_item:
                    result += '%s_LOOKUP' % field.field_name
                else:
                    result += '%s."%s"' % (self.table_alias(), field.field_name)
                if order[1]:
                    result += ' DESC'
                result += ', '
        if result:
            result = result[:-2]
            result = ' ORDER BY ' + result
        elif query['__limit']:
            result = ' ORDER BY %s."%s"' % (self.table_alias(), self._primary_key)
        return db_module.set_case(result)

    def get_select_statement(self, query, db_module=None):
        try:
            if db_module is None:
                db_module = self.task.db_module
            field_list = query['__fields']
            if len(field_list):
                fields = [self._field_by_name(field_name) for field_name in field_list]
            else:
                fields = self._fields
            sql = db_module.get_select(query, \
                self.fields_clause(query, fields, db_module),
                self.from_clause(query, fields, db_module) + \
                self.where_clause(query, db_module) + \
                self.group_clause(query, fields, db_module) + \
                self.order_clause(query, db_module),
                fields)
            return sql
        except Exception, e:
            print self.item_name
            print traceback.format_exc()
            raise

    def get_record_count_query(self, query, db_module=None):
        if db_module is None:
            db_module = self.task.db_module
        fields = []
        filters = query['__filters']
        if filters:
            for (field_name, filter_type, value) in filters:
                fields.append(self._field_by_name(field_name))
        sql = 'SELECT COUNT(*) FROM %s %s' % (self.from_clause(query, fields, db_module),
            self.where_clause(query, db_module))
        return sql

    def create_table_sql(self, db_type, table_name, fields=None, foreign_fields=None):
        if not fields:
            fields = []
            for field in self.fields:
                if not (field.calculated or field.master_field):
                    dic = {}
                    dic['id'] = field.ID
                    dic['field_name'] = field.field_name
                    dic['data_type'] = field.data_type
                    dic['size'] = field.field_size
                    dic['default_value'] = field.f_default_value.value
                    dic['primary_key'] = field.id.value == item.f_primary_key.value
                    fields.append(dic)
        result = []
        db_module = db_modules.get_db_module(db_type)
        result = db_module.create_table_sql(table_name, fields, foreign_fields)
        for i, s in enumerate(result):
            print result[i]
        return result

    def delete_table_sql(self, db_type):
        table_name = self.f_table_name.value
        db_module = db_modules.get_db_module(db_type)
        result = db_module.delete_table_sql(table_name)
        for i, s in enumerate(result):
            print result[i]
        return result

    def recreate_table_sql(self, db_type, old_fields, new_fields, fk_delta=None):

        def foreign_key_dict(ind):
            fields = ind.task.sys_fields.copy()
            fields.set_where(id=ind.f_foreign_field.value)
            fields.open()
            dic = {}
            dic['key'] = fields.f_field_name.value
            ref_id = fields.f_object.value
            items = self.task.sys_items.copy()
            items.set_where(id=ref_id)
            items.open()
            dic['ref'] = items.f_table_name.value
            primary_key = items.f_primary_key.value
            fields.set_where(id=primary_key)
            fields.open()
            dic['primary_key'] = fields.f_field_name.value
            return dic

        def get_foreign_fields():
            indices = self.task.sys_indices.copy()
            indices.filters.owner_rec_id.value = self.id.value
            indices.open()
            del_id = None
            if fk_delta and (fk_delta.rec_modified() or fk_delta.rec_deleted()):
                del_id = fk_delta.id.value
            result = []
            for ind in indices:
                if ind.f_foreign_index.value:
                    if not del_id or ind.id.value != del_id:
                        result.append(foreign_key_dict(ind))
            if fk_delta and (fk_delta.rec_inserted() or fk_delta.rec_modified()):
                result.append(foreign_key_dict(fk_delta))
            return result

        def create_indices_sql(db_type):
            indices = self.task.sys_indices.copy()
            indices.filters.owner_rec_id.value = self.id.value
            indices.open()
            result = []
            for ind in indices:
                if not ind.f_foreign_index.value:
                    result.append(ind.create_index_sql(db_type, self.f_table_name.value, new_fields=new_fields))
            return result

        def find_field(fields, id_value):
            found = False
            for f in fields:
                if f['id'] == id_value:
                    found = True
                    break
            return found

        def prepare_fields():
            for f in list(new_fields):
                if not find_field(old_fields, f['id']):
                    new_fields.remove(f)
            for f in list(old_fields):
                if not find_field(new_fields, f['id']):
                    old_fields.remove(f)

        table_name = self.f_table_name.value
        result = []
        result.append('ALTER TABLE "%s" RENAME TO Temp' % table_name)
        foreign_fields = get_foreign_fields()
        create_sql = self.create_table_sql(db_type, table_name, new_fields, foreign_fields)
        for sql in create_sql:
            result.append(sql)
        prepare_fields()
        old_field_list = ['"%s"' % field['field_name'] for field in old_fields]
        new_field_list = ['"%s"' % field['field_name'] for field in new_fields]
        result.append('INSERT INTO "%s" (%s) SELECT %s FROM Temp' % (table_name, ', '.join(new_field_list), ', '.join(old_field_list)))
        result.append('DROP TABLE Temp')
        ind_sql = create_indices_sql(db_type)
        for sql in ind_sql:
            result.append(sql)
        return result

    def change_table_sql(self, db_type, old_fields, new_fields):

        def recreate(comp):
            for key, (old_field, new_field) in comp.items():
                if old_field and new_field:
                    if old_field['field_name'] != new_field['field_name']:
                        return True
                    elif old_field['default_value'] != new_field['default_value']:
                        return True
                elif old_field and not new_field:
                    return True

        db_module = db_modules.get_db_module(db_type)
        table_name = self.f_table_name.value
        result = []
        comp = {}
        for field in old_fields:
            comp[field['id']] = [field, None]
        for field in new_fields:
            if comp.get(field['id']):
                comp[field['id']][1] = field
            else:
                if field['id']:
                    comp[field['id']] = [None, field]
                else:
                    comp[field['field_name']] = [None, field]
        if db_type == db_modules.SQLITE and recreate(comp):
            result += self.recreate_table_sql(db_type, old_fields, new_fields)
        else:
            for key, (old_field, new_field) in comp.items():
                if old_field and not new_field and db_type != db_modules.SQLITE:
                    result.append(db_module.del_field_sql(table_name, old_field))
            for key, (old_field, new_field) in comp.items():
                if old_field and new_field and db_type != db_modules.SQLITE:
                    if (old_field['field_name'] != new_field['field_name']) or \
                        (db_module.FIELD_TYPES[old_field['data_type']] != db_module.FIELD_TYPES[new_field['data_type']]) or \
                        (old_field['default_value'] != new_field['default_value']) or \
                        (old_field['size'] != new_field['size']):
                        sql = db_module.change_field_sql(table_name, old_field, new_field)
                        if type(sql) in (list, tuple):
                            result += sql
                        else:
                            result.append()
            for key, (old_field, new_field) in comp.items():
                if not old_field and new_field:
                    result.append(db_module.add_field_sql(table_name, new_field))
        for i, s in enumerate(result):
            print result[i]
        return result

    def create_index_sql(self, db_type, table_name, fields=None, new_fields=None, foreign_key_dict=None):

        def new_field_name_by_id(id_value):
            for f in new_fields:
                if f['id'] == id_value:
                    return f['field_name']

        db_module = db_modules.get_db_module(db_type)
        index_name = self.f_index_name.value
        if self.f_foreign_index.value:
            if foreign_key_dict:
                key = foreign_key_dict['key']
                ref = foreign_key_dict['ref']
                primary_key = foreign_key_dict['primary_key']
            else:
                fields = self.task.sys_fields.copy()
                fields.set_where(id=self.f_foreign_field.value)
                fields.open()
                key = fields.f_field_name.value
                ref_id = fields.f_object.value
                items = self.task.sys_items.copy()
                items.set_where(id=ref_id)
                items.open()
                ref = items.f_table_name.value
                primary_key = items.f_primary_key.value
                fields.set_where(id=primary_key)
                fields.open()
                primary_key = fields.f_field_name.value
            sql = db_module.create_foreign_index_sql(table_name, index_name, key, ref, primary_key)
        else:
            index_fields = self.f_fields.value
            desc = ''
            if self.descending.value:
                desc = 'DESC'
            unique = ''
            if self.f_unique_index.value:
                unique = 'UNIQUE'
            fields = common.load_index_fields(index_fields)
            if new_fields:
                fields = [new_field_name_by_id(field[0]) for field in fields]
            else:
                fields = [self.task.sys_fields.field_by_id(field[0], 'f_field_name') for field in fields]
            if desc and db_type in (db_modules.SQLITE, db_modules.POSTGRESQL):
                fields = ['"%s" %s' % (field, desc) for field in fields]
                field_str = ', '.join(fields)
            else:
                field_str = '"' + '", "'.join(fields) + '"'
            sql = db_module.create_index_sql(index_name, table_name, unique, field_str, desc)
        print sql
        return sql

    def delete_index_sql(self, db_type):
        db_module = db_modules.get_db_module(db_type)
        table_name = self.task.sys_items.field_by_id(self.owner_rec_id.value, 'f_table_name')
        index_name = self.f_index_name.value
        if self.f_foreign_index.value:
            sql = db_module.delete_foreign_index(table_name, index_name)
        else:
            sql = db_module.delete_index(table_name, index_name)
        print sql
        return sql



# -*- coding: utf-8 -*-

import sqlite3

NEED_DATABASE_NAME = True
NEED_LOGIN = False
NEED_PASSWORD = False
NEED_ENCODING = False
NEED_HOST = False
NEED_PORT = False
CAN_CHANGE_TYPE = False
CAN_CHANGE_SIZE = True
UPPER_CASE = True
DDL_ROLLBACK = False
FROM = '"%s" AS %s'
LEFT_OUTER_JOIN = 'OUTER LEFT JOIN "%s" AS %s'
FIELD_AS = 'AS'
LIKE = 'LIKE'

JAM_TYPES = TEXT, INTEGER, FLOAT, CURRENCY, DATE, DATETIME, BOOLEAN, BLOB = range(1, 9)
FIELD_TYPES = {
    INTEGER: 'INTEGER',
    TEXT: 'TEXT',
    FLOAT: 'REAL',
    CURRENCY: 'REAL',
    DATE: 'TEXT',
    DATETIME: 'TEXT',
    BOOLEAN: 'INTEGER',
    BLOB: 'BLOB'
}

def sqlite_lower(value_):
    try:
        return value_.lower()
    except:
        pass

def sqlite_upper(value_):
    try:
        return value_.upper()
    except:
        pass

def connect(database, user, password, host, port, encoding):
    connection = sqlite3.connect(database)
    connection.create_function("LOWER", 1, sqlite_lower)
    connection.create_function("UPPER", 1, sqlite_upper)
    cursor = connection.cursor()
    cursor.execute("PRAGMA foreign_keys = ON")
    return connection

def get_lastrowid(cursor):
    return cursor.lastrowid

def get_select(query, start, end, fields):
    offset = query['__offset']
    limit = query['__limit']
    result = 'SELECT %s FROM %s' % (start, end)
    if limit:
        result += ' LIMIT %d, %d' % (offset, limit)
    return result

def process_sql_params(params, cursor):
    result = []
    for p in params:
        if type(p) == tuple:
            value, data_type = p
        else:
            value = p
        result.append(value)
    return result

def process_sql_result(rows):
    result = []
    for row in rows:
        result.append(list(row))
    return result

def cast_date(date_str):
    return "CAST('" + date_str + "' AS DATE)"

def cast_datetime(datetime_str):
    return "CAST('" + datetime_str + "' AS TIMESTAMP)"

def value_literal(index):
    return '?'

def upper_function():
    pass

def create_table_sql(table_name, fields, foreign_fields=None):
    result = []
    primary_key = ''
    sql = set_case('CREATE TABLE "%s"\n(\n' % table_name)
    for field in fields:
        sql += set_case('"%s" %s' % (field['field_name'], FIELD_TYPES[field['data_type']]))
        if field['primary_key']:
            primary_key = set_case(field['field_name'])
            sql += ' PRIMARY KEY'
        if field['default_value']:
            if field['data_type'] == TEXT:
                sql += " DEFAULT '%s'" % field['default_value']
            else:
                sql += ' DEFAULT %s' % field['default_value']
        sql +=  ',\n'
    if foreign_fields:
        for field in foreign_fields:
            sql += set_case('FOREIGN KEY(%s) REFERENCES %s(%s),\n' % (field['key'], field['ref'], field['primary_key']))
    sql = sql[:-2]
    sql += ')\n'
    result.append(sql)
    return result

def delete_table_sql(table_name):
    result = []
    result.append(set_case('DROP TABLE "%s"' % table_name))
    return result

def create_index_sql(index_name, table_name, unique, fields, desc):
    return set_case('CREATE %s INDEX "%s" ON "%s" (%s)' % (unique, index_name, table_name, fields))

def create_foreign_index_sql(table_name, index_name, key, ref):
    return ''

def delete_index(table_name, index_name):
    return set_case('DROP INDEX "%s"' % index_name)

def delete_foreign_index(table_name, index_name):
    pass

def add_field_sql(table_name, field):
    result = set_case('ALTER TABLE "%s" ADD COLUMN "%s" %s' % \
        (table_name, field['field_name'], FIELD_TYPES[field['data_type']]))
    if field['default_value']:
        if field['data_type'] == TEXT:
            sql += " DEFAULT '%s'" % field['default_value']
        else:
            sql += ' DEFAULT %s' % field['default_value']
    return result

def del_field_sql(table_name, field):
    return ''

def change_field_sql(table_name, old_field, new_field):
    return ''

def set_case(string):
    return string.upper()

def get_sequence_name(table_name):
    return None

def next_sequence_value_sql(table_name):
    return None

def restart_sequence_sql(table_name, value):
    pass






# -*- coding: utf-8 -*-

dictionary = \
{
    'lang':                             u'hu',
#admin fields
    'admin':                            u'Adminisztrátor',
    'catalogs':                         u'Katalógusok',
    'journals':                         u'Naplók',
    'tables':                           u'Táblák',
    'reports':                          u'Riportok',
    'details':                          u'Részletek',
    'id':                               u'Rekord Azon',
    'deleted_flag':                     u'Töröltve',
    'caption':                          u'Fejléc',
    'name':                             u'Név',
    'table_field_name':                 u'Field name',
    'table_name':                       u'Tábla',
    'template':                         u'Riport sablon',
    'params_template':                  u'Paraméter sablon',
    'view_template':                    u'Nézet sablon',
    'visible':                          u'Látható',
    'client_module':                    u'Kliens modul',
    'server_module':                    u'Szerver modul',
    'project':                          u'Projekt',
    'users':                            u'Felhasználók',
    'roles':                            u'Szerepek',
    'privileges':                       u'Jogok',
    'tasks':                            u'Task',
    'safe_mode':                        u'Safe mód',
    'language':                         u'Nyelv',
    'author':                           u'Szerző',
    'interface':                        u'Interface',
    'db_type':                          u'DB tipus',
    'db_name':                          u'File név',
    'alias':                            u'Adatbázis',
    'data_type':                        u'Tipus',
    'filter_type':                      u'Szűrő tipus',
    'size':                             u'Máret',
    'object':                           u'Lookup tábla',
    'object_field':                     u'Lookup oszlop',
    'lookup_values':                    u'Lookup érték',
    'master_field':                     u'Mester oszlop',
    'required':                         u'Kötelező',
    'calculated':                       u'Számolt.',
    'default':                          u'Alap',
    'read_only':                        u'Csak olvasható',
    'alignment':                        u'Igazítás',
    'active':                           u'Aktív',
    'date':                             u'Dátum',
    'role':                             u'Szerep',
    'info':                             u'Információ',
    'item':                             u'Item',
    'can_view':                         u'Nézhet',
    'can_create':                       u'Létrehozhat',
    'can_edit':                         u'Szerkeszthet',
    'can_delete':                       u'Törölhet',
    'fields':                           u'Mezők',
    'field':                            u'Mező',
    'filter':                           u'Szűrő',
    'filters':                          u'Szűrők',
    'index':                            u'Index',
    'index_name':                       u'Index név',
    'report_params':                    u'Riport paraméter',
    'error':                            u'Hiba',
#admin interface
    'db':                               u'Adatbázis',
    'export':                           u'Export',
    'import':                           u'Import',
    'viewing':                          u'Megnéz',
    'editing':                          u'Szerkeszt',
    'filters':                          u'Szűrők',
    'order':                            u'Sorrend',
    'indices':                          u'Indexek',
    'foreign_keys':                     u'Foreign keys',
    'select_all':                       u'Kijelöl mind',
    'unselect_all':                     u'Semmit jelöl',
    'project_params':                   u'Paraméterek',
    'project_locale':                   u'Területi',
    'reserved_word':                    u'A nev foglalt kulcsszó',
    'is_edited':                        u'%s is already open for editing. Nevertheless, open it?',
    'is_edited_by':                     u'%s is already open for editing by %s. Nevertheless, open it?',
#editor
    'case_sensitive':                   u'Betü érzékeny',
    'whole_words':                      u'Teljesz szóra keres',
    'in_task':                          u'a taskban',
    'text_not_found':                   u'Nem található.\nBontsa meg és újra?',
    'text_changed':                     u'A modul megváltozott.\nBezárás elött legyen mentés?',
    'go_to_line':                       u'Ugrás a sorhoz',
    'go_to':                            u'Ugrás',
    'line':                             u'sor',
#admin editors
    'caption_name':                     u'Név',
    'caption_word_wrap':                u'Tördelás',
    'caption_expand':                   u'Bővit',
    'caption_edit':                     u'Szerkeszt',
    'caption_descening':                u'Csök.',
#admin messages
    'fill_task_attrubutes':             u'Töltse ki a címet és az adat bázis tipusát.',
    'can_not_connect':                  u"%s adat bázishoz nem lehet csatlakozni.",
    'field_used_in_filters':            u"%s mező nem törölhető.\n A szűrő definoció használja:\n%s",
    'field_used_in_fields':             u"%s mező nem törölhető.\n A mező definoció használja:\n%s",
    'field_used_in_indices':            u"%s mező nem törölhető.\n Az index definoció használja:\n%s",
    'field_is_system':                  u"Rendszer mező nem törölhető.",
    'detail_mess':                      u'%s - részlete %s',
    'item_used_in_items':               u"%s tétel nem törölhető.\n A tétel definició használja:\n%s",
    'field_mess':                       u'%s - mező %s',
    'item_used_in_fields':              u"s tétel nem törölhető.\n A mező definició használja:\n%s",
    'param_mess':                       u'%s - paraméter %s',
    'item_used_in_params':              u"s tétel nem törölhető.\n A paraméter definició használja:\n%s",
    'invalid_name':                     u'Érvénytelen név.',
    'invalid_field_name':               u'Érvénytelen mezőnév.',
    'type_is_required':                 u'A mező tipus kötelező.',
    'index_name_required':              u'Az index név kötelező.',
    'index_fields_required':            u'Az index mező kötelező.',
    'cant_delete_group':                u"Csoport nem törölhető.",
    'object_field_required':            u'Lookup mező szükséges',
    'no_tasks_ptoject':                 u'A project nem tartalmaz taskot.',
    'stop_server':                      u'Szerver álj',
#interface buttons and labels
    'yes':                              u'Igen',
    'no':                               u'Nem',
    'ok':                               u'OK',
    'cancel':                           u'Mégsem',
    'delete':                           u'Töröl',
    'new':                              u'Új',
    'edit':                             u'Szerkeszt',
    'copy':                             u'Másol',
    'print':                            u'Print',
    'save':                             u'Ment',
    'open':                             u'Megnyit',
    'close':                            u'Bezár',
    'select':                           u'Kiválaszt',
    'filter':                           u'Szűrő',
    'apply':                            u'Alkalmaz',
    'find':                             u'Keres',
    'replace':                          u'Cserél',
    'view':                             u'megnéz',
    'log_in':                           u'Belép',
    'login':                            u'Belépés',
    'password':                         u'Jelszó',
    'log_out':                          u'Kilép',
#runtime messages
    'invalid_int':                      u'%s rossz érték - egésznek kell lenni',
    'invalid_float':                    u'%s rossz érték - lebegőpontosnak kell lenni',
    'invalid_cur':                      u'%s rossz érték - pénznek kell lenni',
    'invalid_date':                     u'%s rossz érték - dátumnak kell lenni',
    'invalid_bool':                     u'%s rossz érték - logikainak kell lenni',
    'invalid_value':                    u'%s rossz érték',
    'value_required':                   u'Nem lehet üres',
    'invalid_length':                   u'A szöveg tul hosszú, maximum hossz - %s',
    'save_changes':                     u'Módosítás történt. Legyen mentés?',
    'delete_record':                    u'Töröljük a rekordot?',
    'no_record':                        u'Record is not selected.',
    'server_request_error':             u'Szerver kérelmi hiba',
    'cant_delete_used_record':          u"A rekord nem törölhető használatban van.",
    'website_maintenance':              u"A szerver ideglenesen nem elérhető.",
    'server_restarted':                 u"Server restarted. Press F5 to reload the page.",
    'version_changed':                  u"Verzió váltás F5 a frissítésekhez",
    'no_task':                          u'The project is loading or the creation of the project has not been completed yet.',
    'items_selected':                   u"Kiválasztva %s tétel",
    'invalid_range':                    u'Index hiba',
    'range_from':                       u'tól',
    'range_to':                         u'ig',
#rights messages
    'cant_view':                        u'%s: nincs jogosultjága megnézni.',
    'cant_create':                      u'%s: nincs jogosultjága létrehozni.',
    'cant_edit':                        u'%s: nincs jogosultjága szerkeszteni.',
    'cant_delete':                      u'%s: nincs jogosultjága törölni.',
#calendar
    'week_start':                        0,
    'days_min':                         [u'V', u'H', u'Ke', u'Sze', u'Cs', u'P', u'Szo', u'V'],
    'months':                           [u'Január', u'Február', u'Március', u'Április', u'Május', u'Junius', u'Julius', u'Augusztus', u'Szeptember', u'Oktober', u'November', u'December'],
    'months_short':                     [u'Jan', u'Feb', u'Már', u'Ápr', u'Máj', u'Jun', u'Jul', u'Aug', u'Sep', u'Okt', u'Nov', u'Dec'],
#grid
    'page':                             u'Lap',
    'of':                               u'a',
#history
    'created':                        u'Created',
    'modified':                       u'Modified',
    'deleted':                        u'Deleted',
    'by_user':                        u'by',
    'old_value':                      u'old value',
    'new_value':                      u'new value'

}



# -*- coding: utf-8 -*-

dictionary = \
{
    'lang':                             u'pt',
#admin fields
    'admin':                            u'Administrador',
    'catalogs':                         u'Catalogos',
    'journals':                         u'Transações',
    'tables':                           u'Tabelas',
    'reports':                          u'Relatórios',
    'details':                          u'Detalhes',
    'id':                               u'ID de Registro',
    'deleted_flag':                     u'Excluído?',
    'caption':                          u'Título',
    'name':                             u'Nome',
    'table_field_name':                 u'Field name',
    'table_name':                       u'Nome da Tabela',
    'template':                         u'Relatório (modelo)',
    'params_template':                  u'Parâmetros',
    'view_template':                    u'Relatório (modelo)',
    'visible':                          u'Visível',
    'client_module':                    u'Cliente (módulo)',
    'server_module':                    u'Servidor (módulo)',
    'project':                          u'Projeto',
    'users':                            u'Usuários',
    'roles':                            u'Perfis',
    'privileges':                       u'Privilégios',
    'tasks':                            u'Tarefas',
    'safe_mode':                        u'Modo Seguro',
    'language':                         u'Idioma',
    'author':                           u'Autor',
    'interface':                        u'Interface',
    'db_type':                          u'BD Tipo',
    'db_name':                          u'Nome do Arquivo',
    'alias':                            u'Banco de Dados',
    'data_type':                        u'Tipo',
    'filter_type':                      u'Tipo de Filtro',
    'size':                             u'Tamanho',
    'object':                           u'Item da Pesquisa',
    'object_field':                     u'Campo da Pesquisa',
    'lookup_values':                    u'Valor da Pesquisa',
    'master_field':                     u'Campo Principal',
    'required':                         u'Obrigatório',
    'calculated':                       u'Calculado',
    'default':                          u'Padrão',
    'read_only':                        u'Somente Leitura',
    'alignment':                        u'Alinhamento',
    'active':                           u'Ativo',
    'date':                             u'Data',
    'role':                             u'Perfil',
    'info':                             u'Informação',
    'item':                             u'Item',
    'can_view':                         u'Pode ver',
    'can_create':                       u'Pode criar',
    'can_edit':                         u'Pode editar',
    'can_delete':                       u'Pode excluir',
    'fields':                           u'Campos',
    'field':                            u'Campo',
    'filter':                           u'Filtro',
    'filters':                          u'Filtros',
    'index':                            u'Índice',
    'index_name':                       u'Nome do Índice',
    'report_params':                    u'Parâmetros do Relatório',
    'error':                            u'Erro',
#admin interface
    'db':                               u'Banco de Dados',
    'export':                           u'Exportar',
    'import':                           u'Importar',
    'viewing':                          u'Ver',
    'editing':                          u'Editar',
    'filters':                          u'Filtros',
    'order':                            u'Ordem',
    'indices':                          u'Índices',
    'foreign_keys':                     u'Change Extrangeira',
    'select_all':                       u'Selecionar tudo',
    'unselect_all':                     u'Deselecionar tudo',
    'project_params':                   u'Parâmetros',
    'project_locale':                   u'Localidade',
    'reserved_word':                    u'O nome é uma palavra reservada',
    'is_edited':                        u'%s is already open for editing. Nevertheless, open it?',
    'is_edited_by':                     u'%s is already open for editing by %s. Nevertheless, open it?',
#editor
    'case_sensitive':                   u'Diferenciar Maiúsculas e Minúsculas',
    'whole_words':                      u'Concidir palavras inteiras',
    'in_task':                          u'Na Tarefa',
    'text_not_found':                   u'Texto não encontrado.\nModificar a busca e tentar novamente?',
    'text_changed':                     u'O Módulo foi modificado.\nSalvar antes de fechar?',
    'go_to_line':                       u'Ir para linha',
    'go_to':                            u'Ir para',
    'line':                             u'Linha',
#admin editors
    'caption_name':                     u'Nome',
    'caption_word_wrap':                u'Cortar',
    'caption_expand':                   u'Expandir',
    'caption_edit':                     u'Editar',
    'caption_descening':                u'Descendente',
#admin messages
    'fill_task_attrubutes':             u'Preencher nos atributos de título, nome e banco de dados.',
    'can_not_connect':                  u"Não foi possível conectar ao banco de dados. %s",
    'field_used_in_filters':            u"Não foi possível deletar o campo %s.\n Está sendo utilizado nas definições de filtro:\n%s",
    'field_used_in_fields':             u"Não foi possível deletar o campo %s.\n Está sendo utilizado nas definições de campo:\n%s",
    'field_used_in_indices':            u"Não foi possível deletar o campo %s.\n Está sendo utilizado nas definições de índice:\n%s",
    'field_is_system':                  u"Não é possível deletar um campo de padrão.",
    'detail_mess':                      u'%s - detalhe %s',
    'item_used_in_items':               u"Nao foi possível deletar o item %s.\n Está sendo utilizado nas definições de item:\n%s",
    'field_mess':                       u'%s - campo %s',
    'item_used_in_fields':              u"Não foi possível deletar o item %s.\n Está sendo utilizado nas definições de campo:\n%s",
    'param_mess':                       u'%s - parâmetros %s',
    'item_used_in_params':              u"Não foi possível deletar o item %s.\n Está sendo utilizado nas definições de parâmetros:\n%s",
    'invalid_name':                     u'o Nome é inválido.',
    'invalid_field_name':               u'O nome do campo é inválido.',
    'type_is_required':                 u'O tipo do campo é requerido.',
    'index_name_required':              u'O índice é requerido.',
    'index_fields_required':            u'Não há campos de índice requeridos.',
    'cant_delete_group':                u"Não é possível deletar um grupo.",
    'object_field_required':            u'Um campo de pesquisa é obrigatório.',
    'no_tasks_ptoject':                 u'Não existem tarefas no projeto.',
    'stop_server':                      u'Parar servidor',
#interface buttons and labels
    'yes':                              u'Sim',
    'no':                               u'Não',
    'ok':                               u'OK',
    'cancel':                           u'Cancelar',
    'delete':                           u'Excluir',
    'new':                              u'Novo',
    'edit':                             u'Editar',
    'copy':                             u'Copiar',
    'print':                            u'Imprimir',
    'save':                             u'Guardar',
    'open':                             u'Abrir',
    'close':                            u'Fechar',
    'select':                           u'Selecionar',
    'filter':                           u'Filtrar',
    'apply':                            u'Aplicar',
    'find':                             u'Pesquisar',
    'replace':                          u'Substituir',
    'view':                             u'Ver',
    'log_in':                           u'Logar',
    'login':                            u'Usuário',
    'password':                         u'Senha',
    'log_out':                          u'Sair',
#runtime messages
    'invalid_int':                      u'%s valor inválido - deve ser um inteiro',
    'invalid_float':                    u'%s valor inválido - deve ser um decimal',
    'invalid_cur':                      u'%s valor inválido - deve ser uma moeda',
    'invalid_date':                     u'%s valor inválido - deve ser uma data',
    'invalid_bool':                     u'%s valor inválido - deve ser um boleano',
    'invalid_value':                    u'%s valor inválido',
    'value_required':                   u'um valor é requerido',
    'invalid_length':                   u'O texto ultrapassa o tamanho máximo permitido - %s',
    'save_changes':                     u'Os dados foram alterados. Você gostaria de salvar as modificações?',
    'delete_record':                    u'Deletar o registro?',
    'no_record':                        u'Record is not selected.',
    'server_request_error':             u'Erro de requisição ao servidor',
    'cant_delete_used_record':          u"Impossível deletar o registro. Ele está sendo usado.",
    'website_maintenance':              u"O servidor está temporáriamente indisponível.",
    'server_restarted':                 u"Server restarted. Press F5 to reload the page.",
    'version_changed':                  u"Versão modificada. Pressione F5 para atualizar",
    'no_task':                          u'The project is loading or the creation of the project has not been completed yet.',
    'items_selected':                   u"selecionado: %s",
    'invalid_range':                    u'extensão invalida',
    'range_from':                       u'de',
    'range_to':                         u'para',
#rights messages
    'cant_view':                        u'%s: você não tem permissões para ver',
    'cant_create':                      u'%s: você não tem permissões para criar',
    'cant_edit':                        u'%s: você não tem permissões para editar',
    'cant_delete':                      u'%s: você não tem permissões para excluir',
#calendar
    'week_start':                        0,
    'days_min':                         [u'Dom', u'Seg', u'Ter', u'Qua', u'Qui', u'Sex', u'Sáb', u'Dom'],
    'months':                           [u'Janeiro', u'Fevereiro', u'Março', u'Abril', u'Maio', u'Junho', u'Julho', u'Agosto', u'Setembro', u'Outubro', u'Novembro', u'Dezembro'],
    'months_short':                     [u'Jan', u'Fev', u'Mar', u'Abr', u'Mai', u'Jun', u'Jul', u'Ago', u'Set', u'Out', u'Nov', u'Dez'],
#grid
    'page':                             u'Página',
    'of':                               u'de',
#history
    'created':                        u'Created',
    'modified':                       u'Modified',
    'deleted':                        u'Deleted',
    'by_user':                        u'by',
    'old_value':                      u'old value',
    'new_value':                      u'new value'
}
